#!/bin/bash
#
# Reloads Web Safety and Squid Proxy, usually called by the Web UI
#
# returns:
#        0 - on success
#       !0 - on various errors
#

# set path to executables
SYSTEMCTL=/bin/systemctl
SQUID_NAME=squid

# adjust paths if needed for centos 7
if [ -f /etc/centos-release ] || [ -f /etc/redhat-release ] || [ -f /etc/SuSE-release ]; then
    SYSTEMCTL=/usr/bin/systemctl
fi

# construct squid path
SQUID_PATH=/usr/sbin/$SQUID_NAME

#
# see if wsgsbd daemon is running, if not reload otherwise restart (note we do NOT check for successful restart/reload!)
#
$SYSTEMCTL status wsgsbd

res=$?
if [ $res -ne 0 ]; then
    echo "Web Safety Google Safe Browsing daemon is not running, restarting it..."

    $SYSTEMCTL stop wsgsbd 2>&1 || true
    $SYSTEMCTL start wsgsbd 2>&1
    
else
    echo "reloading Web Safety Google Safe Browsing daemon..."
    
    $SYSTEMCTL reload wsgsbd 2>&1
    res=$?
    if [ $res -ne 0 ]; then
        echo "cannot reload - error $res, restarting it ..."

        $SYSTEMCTL stop wsgsbd 2>&1 || true
        $SYSTEMCTL start wsgsbd 2>&1
    fi
fi

#
# see if wsytgd daemon is running, if not reload otherwise restart (note we do NOT check for successful restart/reload!)
#
$SYSTEMCTL status wsytgd

res=$?
if [ $res -ne 0 ]; then
    echo "Web Safety YouTube Guard daemon is not running, restarting it..."

    $SYSTEMCTL stop wsytgd 2>&1 || true
    $SYSTEMCTL start wsytgd 2>&1
    
else
    echo "reloading Web Safety YouTube Guard daemon..."
    
    $SYSTEMCTL reload wsytgd 2>&1
    res=$?
    if [ $res -ne 0 ]; then
        echo "cannot reload - error $res, restarting it ..."

        $SYSTEMCTL stop wsytgd 2>&1 || true
        $SYSTEMCTL start wsytgd 2>&1
    fi
fi

#
# see if wsmgrd daemon is running, if not reload otherwise restart
#
$SYSTEMCTL status wsmgrd

res=$?
if [ $res -ne 0 ]; then
    echo "Web Safety Manager daemon is not running, restarting it..."

    $SYSTEMCTL stop wsmgrd 2>&1 || true
    $SYSTEMCTL start wsmgrd 2>&1
    res=$?
    if [ $res -ne 0 ]; then
        echo "cannot restart, error $res"
        exit 1
    fi
else
    echo "reloading Web Safety Manager daemon..."
    
    $SYSTEMCTL reload wsmgrd 2>&1
    res=$?
    if [ $res -ne 0 ]; then
        echo "cannot reload - error $res, restarting it ..."

        $SYSTEMCTL stop wsmgrd 2>&1 || true
        $SYSTEMCTL start wsmgrd 2>&1
        res=$?
        if [ $res -ne 0 ]; then
            echo "cannot restart, error $res"
            exit 2
        fi
    fi
fi

# see if wsicapd daemon is running if not reload otherwise restart
$SYSTEMCTL status wsicapd

res=$?
if [ $res -ne 0 ]; then
    echo "Web Safety ICAP daemon is not running, restarting it..."

    $SYSTEMCTL stop wsicapd 2>&1 || true
    $SYSTEMCTL start wsicapd 2>&1
    res=$?
    if [ $res -ne 0 ]; then
        echo "cannot restart, error $res"
        exit 3
    fi
else
    echo "reloading Web Safety ICAP daemon..."
    
    $SYSTEMCTL reload wsicapd 2>&1
    res=$?
    if [ $res -ne 0 ]; then
        echo "cannot reload - error $res, restarting it ..."

        $SYSTEMCTL stop wsicapd 2>&1 || true
        $SYSTEMCTL start wsicapd 2>&1
        res=$?
        if [ $res -ne 0 ]; then
            echo "cannot restart, error $res"
            exit 4
        fi
    fi
fi

# verify squid configuration
echo "verifying Squid configuration file..."
$SQUID_PATH -k parse 2>&1
res=$?
if [ $res -ne 0 ]; then     
    echo "configuration file cannot be verified, error $res"
    exit 5
fi


# and finally reload squid
echo "reloading Squid proxy..."

$SYSTEMCTL reload $SQUID_NAME 2>&1
res=$?
if [ $res -ne 0 ]; then     
    echo "reload failed with error $?, trying to restart..."

    $SYSTEMCTL stop $SQUID_NAME 2>&1 || true
    $SYSTEMCTL start $SQUID_NAME 2>&1    
    res=$?
    if [ $res -ne 0 ]; then     
        echo "cannot restart squid, error $res"
        exit 6
    fi
fi

# dump success
echo "reload successful!"
exit 0
