#!/bin/bash
#
# Restarts Web Safety and Squid Proxy, usually called by the Web UI
#
# returns:
#       0 - on success
#      !0 - on various errors
#

# set path to executables
SYSTEMCTL=/bin/systemctl
SQUID_NAME=squid

# adjust paths if needed for centos 7
if [ -f /etc/centos-release ] || [ -f /etc/redhat-release ] || [ -f /etc/SuSE-release ]; then
    SYSTEMCTL=/usr/bin/systemctl    
fi

# construct squid path
SQUID_PATH=/usr/sbin/$SQUID_NAME

# restart wsgsbd (do NOT check for success!)
echo "restarting Web Safety Google Safe Browsing daemon..."
$SYSTEMCTL stop wsgsbd 2>&1 || true
$SYSTEMCTL start wsgsbd

# restart wsytgd (do NOT check for success!)
echo "restarting Web Safety YouTube Guard daemon..."
$SYSTEMCTL stop wsytgd 2>&1 || true
$SYSTEMCTL start wsytgd

# restart wsicapd
echo "restarting Web Safety ICAP daemon..."
$SYSTEMCTL stop wsicapd 2>&1 || true
$SYSTEMCTL start wsicapd
res=$?
if [ $res -ne 0 ]; then
    echo "cannot restart, error $res"
    exit 1
fi

# verify squid configuration
echo "verifying Squid configuration file..."
$SQUID_PATH -k parse 2>&1
res=$?
if [ $res -ne 0 ]; then     
    echo "configuration file cannot be verified, error $res"
    exit 2
fi

# restart squid
echo "restarting Squid proxy..."
$SYSTEMCTL stop $SQUID_NAME 2>&1 || true
$SYSTEMCTL start $SQUID_NAME
res=$?
if [ $res -ne 0 ]; then     
    echo "cannot restart squid, error $res"
    exit 3
fi

# restart wsmgrd
echo "restarting Web Safety Manager..."
$SYSTEMCTL stop wsmgrd 2>&1 || true
$SYSTEMCTL start wsmgrd
res=$?
if [ $res -ne 0 ]; then
    echo "cannot restart, error $res"
    exit 4
fi

# restart wssyncd (do NOT check for success!)
echo "restarting Web Safety Sync ..."
$SYSTEMCTL stop wssyncd 2>&1 || true
$SYSTEMCTL start wssyncd

# show status
$SYSTEMCTL status wsicapd
$SYSTEMCTL status $SQUID_NAME
$SYSTEMCTL status wsmgrd
$SYSTEMCTL status wsgsbd

# dump success
echo "restart successful!"
exit 0
