import os

#
#
#
from _domain.core import System, Paths
from _domain.utils import FileReader, CommandTail

#
#
#
class ErrorLog:

    def get(self):

        path = os.path.join(Paths.var_dir(), "log", "error.log")
        try:
            return FileReader(path).read()
        except Exception as e:
            return "%s does not exist or is not accessible, error - %s. It might be the process did not write anything to the log yet. Try to refresh this page in a while." % (path, str(e))

#
#
#
class UpdateLog:

    def get(self):
        
        path = os.path.join(Paths.var_dir(), "log", "cron_update.log")
        try:
            return FileReader(path).read()
        except Exception as e:
            return "%s does not exist or is not accessible, error - %s. It might be the process did not write anything to the log yet. Try to refresh this page in a while." % (path, str(e))

#
#
#
class WsYtgdLogDir:

    @staticmethod
    def get():
        
        if System.WS_FREEBSD == System.name():
            return os.path.join(Paths.var_dir(), "log")
        else:
            return os.path.join("/", "var", "log")

#
#
#
class WsYtgdLog:

    def __init__(self):

        self.dir = WsYtgdLogDir.get()
        self.log = os.path.join(self.dir, "wsytgd.log")

    def get_path(self):
        
        return self.log

    def get_size(self):

        size = 0
        if os.path.exists(self.log):
            size = os.path.getsize(self.log)

        return size

    def get_contents(self, line_count, include, exclude):

        (code, stdout, stderr) = CommandTail().run(self.log, line_count)
        if 0 != code:
            return stderr

        # got output, do exclusions
        if len(exclude) > 0:
            lines = []
            for line in stdout.split('\n'):
                if line.find(exclude) == -1:
                    lines.append(line)

            stdout = '\n'.join(lines)

        # do inclusions
        if len(include) > 0:
            lines = []
            for line in stdout.split('\n'):
                if line.find(include) != -1:
                    lines.append(line)
            stdout = '\n'.join(lines)

        return stdout
