import json
import traceback

#
#
#
from _domain.core import Paths, WsUser
from _domain.safety import WsYtgdStatus

#
# django
#
from django.views import generic
from django.http import HttpResponse

#
#
#
from safety.models import YouTube
from .api_ps import CommandPs

#
#
#
class WsytgdInfo(object):

    def __init__(self):
        
        self.path = ""
        self.pid  = ""
        self.user = ""

#
#
#
class WsytgdStats(object):

    def __init__(self):
        
        self.cpu_time   = ""
        self.cpu_usage  = ""
        self.mem_size   = 0
        self.mem_usage  = ""

#
#
#
class ViewApiDashWsytgd(generic.View):

    def squid_exe_found(self, line):

        if line.find("squid-1") != -1:
            return True

        full_path = BinarySquid.full_path()
        if line.find(full_path) != -1:
            return True

        return False

    def get_process(self):

        # get processes of wsytgd daemon
        processes = CommandPs(WsUser.name()).run()
        
        # see if wsytgd binary is there and fill the stats
        for process in processes:

            if process['path'].find(Paths.bin_dir() + "/wsytgd") != -1:

                # ok found, as we know the wsytgd process is single per system, fill info
                info = WsytgdInfo()
                if True:
                    info.path = process['path']
                    info.pid  = process['pid']
                    info.user = process['user']

                # fill stats
                stats = WsytgdStats()
                if True:
                    stats.cpu_time  = process['cpu_time']
                    stats.cpu_usage = process['cpu_usage']
                    stats.mem_size  = int(process['mem_size'])
                    stats.mem_usage = process['mem_usage']

                # perfect now return
                return (info, stats, [process])

        # no wsmgrd daemon means something is really bad tell the caller
        raise Exception("The wsytgd daemon is not running")

    def get(self, request, *args, **kwargs):

        # allocate default response
        data = { 'error' : False, 'desc' : '', 'info' : {}, 'stats' : {}, 'runtime' : {} }

        # get daemon information
        try:

            # get process info
            info, stats, processes = self.get_process()

            # convert info
            data['info'] = {
                'path' : info.path,
                'pid'  : info.pid,
                'user' : info.user
            }

            # convert stats to json and store in data
            data['stats'] = {
                'cpu_time'  : stats.cpu_time,
                'cpu_usage' : stats.cpu_usage,
                'mem_size'  : stats.mem_size,
                'mem_usage' : stats.mem_usage
            }

            # and store processes too
            data['processes'] = processes

        except Exception as e:

            data['error']     = True
            data['desc']      = str(e)
            data['traceback'] = "%s" % traceback.format_exc()

        # and store as array
        return HttpResponse(json.dumps([data], ensure_ascii=True), content_type='application/json')

#
#
#
class ViewApiDashWsytgdRuntime(generic.View):

    def get(self, request, *args, **kwargs):

        try:

            youtube = YouTube.objects.first()

            # allocate default info
            data = {
                'info'  : WsYtgdStatus().run(youtube.daemon_port),
                'error' : False,
                'desc'  : ""
            }
            
        except Exception as e:
            
            data = {
                'error'     : True,
                'desc'      : str(e),
                'traceback' : "%s" % traceback.format_exc()
            }
        
        return HttpResponse(json.dumps([data], ensure_ascii=True), content_type='application/json')
