#
#
#
from _domain.core import Paths, Build, Distrib, System
from _domain.safety import ErrorLog, WsYtgdLog
from _domain.utils import Version, FolderInfo

#
#
#
from django.views import generic

#
#
#
class ViewSafetySystemInfo(generic.TemplateView):

    template_name = "safety/support/system.html"

    def get_context_data(self, **kwargs):
        
        context = super(ViewSafetySystemInfo, self).get_context_data(**kwargs)

        v = Version()

        info = {
            'product_name'      : 'Web Safety for Squid Proxy',
            'installed_version' : v.installed,
            'latest_version'    : v.latest,
            'need_to_upgrade'   : v.need_to_upgrade(),        # 0 - no_need_to_upgrade, 1 - may_upgrade, 2 - should_upgrade, 3 - must_upgrade
            'whats_new'         : v.whats_new
        };
        context['info'] = info

        # add hardcoded settings
        context['WEBSAFETY_ETC_DIR']      = Paths.etc_dir()
        context['WEBSAFETY_ETC_DIR_SIZE'] = int(FolderInfo(Paths.etc_dir()).get_size()) 
        context['WEBSAFETY_VAR_DIR']      = Paths.var_dir()
        context['WEBSAFETY_VAR_DIR_SIZE'] = int(FolderInfo(Paths.var_dir()).get_size())
        context['WEBSAFETY_BIN_DIR']      = Paths.bin_dir()
        context['WEBSAFETY_BIN_DIR_SIZE'] = int(FolderInfo(Paths.bin_dir()).get_size())
        context['WEBSAFETY_VERSION']      = Build.version()
        context['WEBSAFETY_ARCH']         = Distrib.arch()
        context['WEBSAFETY_DISTRIB']      = Distrib.name()
        context['WEBSAFETY_SYSTEM']       = System.name()
        
        return context

#
#
#
class ViewSafetyErrorLog(generic.TemplateView):

    template_name = "safety/support/error_log.html"

    def get_context_data(self, **kwargs):
        context = super(ViewSafetyErrorLog, self).get_context_data(**kwargs)
        context['error_log'] = ErrorLog().get()
        return context

#
# 
#
class ViewSafetyYouTubeLog(generic.base.TemplateView):

    template_name = "safety/support/wsytgd_log.html"
    
    def get_context_data(self, **kwargs):

        context = super(ViewSafetyYouTubeLog, self).get_context_data(**kwargs)        

        # get size, include and exclude strings from request
        include = self.request.GET.get('include', '')
        exclude = self.request.GET.get('exclude', '')
        size    = 256
        try:
            size = int(self.request.GET.get('size', '256'))
            if size > 1024:
                size = 1024
        except Exception as e:
            pass

        # save into context
        context['size']         = size
        context['include']      = include
        context['exclude']      = exclude
        context['log_contents'] = self.get_contents(size, include, exclude)
        context['log_size']     = self.get_size()
        context['log_path']     = WsYtgdLog().get_path()

        return context

    def get_size(self):

        log  = WsYtgdLog()
        size = log.get_size()
       
        return size

    def get_contents(self, line_count, include, exclude):

        log      = WsYtgdLog()
        contents = log.get_contents(line_count, include, exclude)
        if not contents:
            contents = "Requested log file is empty."
        
        return contents
