#!/usr/bin/env python
#
# (c) Diladele B.V. 2018, Web Safety for Squid Proxy
#  
# Sample automation script
#
import os
import sys
import django
import logging
import argparse

#
# in order for Django to work correctly with our monitor app, we add ourselves
# to Python's search path
sys.path.append(os.path.abspath(__file__))

#
# tell Django where to read settings from
#
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "console.settings")

#
#
#
django.setup()

#
# do imports
#
from safety.models import *
from squid.models import *
from node.models import *
from node.blogic import NetworkGenerator, NetworkDeviceSyncer

#
#
#
def process_network(network):

    logging.info("Resetting network configuration to '%s' ..." % network)

    item = NetAdvanced.objects.first()

    if network in ["none", "manual"]:
        item.network_type = NetAdvanced.NETWORK_TYPE_NONE
    elif network in ["debian10"]:
        item.network_type = NetAdvanced.NETWORK_TYPE_INTERFACES
    elif network in ["ubuntu18"]:
        item.network_type = NetAdvanced.NETWORK_TYPE_NETPLAN
    else:
        pass
        
    item.save()

    # forcefully reread all NICs from the system
    force_sync = True
    NetworkDeviceSyncer().reread_from_system(force_sync)

    # now generate the contents on disk
    NetworkGenerator().generate(
        NetDevice.objects.all(),
        NetAdvanced.objects.first()
    )

    logging.info("Done, network configuration is now '%s' ..." % network)

    
#
#
#
def process_args(args):

    if args.network is not None:
        process_network(args.network)
    else:
        logging.info("No argument specified, aborting.")

#
# main
#
def main():
    
    # define arguments
    parser = argparse.ArgumentParser(description='Utility script for post build steps for Web Safety')
    parser.add_argument("--network", help="none|debian10|ubuntu18", required=False)

    # parse them
    args = parser.parse_args()    

    logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)   
        
    # process
    process_args(args)

    logging.info("Utility script finished successfully!")
        
if __name__ == '__main__':

    main()